var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};

// src/mdxPrism/mdxPrism.ts
import { toString } from "hast-util-to-string";
import { refractor } from "refractor/lib/all";
import { rehype } from "rehype";
import parse from "rehype-parse";
import { unified } from "unified";
import { visit } from "unist-util-visit";

// src/addMarkers/helpers.ts
import { filter } from "unist-util-filter";
import { visitParents } from "unist-util-visit-parents";
function wrapLines(treeNodes, markers, options) {
  if (markers.length === 0 || treeNodes.length === 0) {
    return treeNodes;
  }
  const ast = markers.reduce((acc, marker) => unwrapLine(marker.line, acc), treeNodes);
  const wrapped = [];
  let astIndex = 0;
  for (let m = 0; m < markers.length; m++) {
    const marker = markers[m];
    for (let node = ast[astIndex]; node && node.lineEnd < marker.line; node = ast[++astIndex]) {
      wrapped.push(node);
    }
    const batch = [];
    for (let node = ast[astIndex]; node && node.lineEnd === marker.line; node = ast[++astIndex]) {
      batch.push(node);
    }
    if (batch.length > 0) {
      wrapped.push(wrapBatch(batch, marker, options));
    }
  }
  while (astIndex < ast.length) {
    wrapped.push(ast[astIndex++]);
  }
  return wrapped;
}
function unwrapLine(markerLine, nodes) {
  const tree = { type: "root", children: nodes };
  const headMap = new Map();
  const lineMap = new Map();
  const tailMap = new Map();
  const cloned = [];
  function addCopy(map, node, ancestors) {
    var _a;
    cloned.push(node);
    ancestors.forEach((ancestor) => {
      if (!map.has(ancestor)) {
        map.set(ancestor, Object.assign({}, ancestor, { children: [] }));
        if (ancestor !== tree) {
          cloned.push(ancestor);
        }
      }
    });
    let i = ancestors.length;
    while (i--) {
      const ancestor = map.get(ancestors[i]);
      const child = ancestors[i + 1];
      const leaf = map.get(child) || node;
      if (((_a = ancestor == null ? void 0 : ancestor.children) == null ? void 0 : _a.indexOf(leaf)) === -1) {
        ancestor.children.push(leaf);
      }
    }
  }
  visitParents(tree, (node, ancestors) => {
    if (node.children) {
      return;
    }
    if (node.lineStart < markerLine) {
      addCopy(headMap, node, ancestors);
      return;
    }
    if (node.lineStart === markerLine) {
      addCopy(lineMap, node, ancestors);
      return;
    }
    if (node.lineEnd > markerLine && cloned.some((clone) => ancestors.indexOf(clone) !== -1)) {
      addCopy(tailMap, node, ancestors);
    }
  });
  const filtered = filter(tree, (node) => cloned.indexOf(node) === -1);
  const getChildren = (map) => {
    const rootNode = map.get(tree);
    if (!rootNode) {
      return [];
    }
    visitParents(rootNode, (leaf, ancestors) => {
      if (leaf.children) {
        leaf.lineStart = 0;
        leaf.lineEnd = 0;
        return;
      }
      ancestors.forEach((ancestor) => {
        ancestor.lineStart = Math.max(ancestor.lineStart, leaf.lineStart);
        ancestor.lineEnd = Math.max(ancestor.lineEnd, leaf.lineEnd);
      });
    });
    return rootNode.children;
  };
  const merged = [].concat(getChildren(headMap), getChildren(lineMap), getChildren(tailMap), filtered ? filtered.children : []);
  headMap.clear();
  lineMap.clear();
  tailMap.clear();
  return merged;
}
function wrapBatch(children, marker, options) {
  var _a, _b;
  const className = marker.className || ((_a = options.lineHighlight) == null ? void 0 : _a.className) || "mdx-marker";
  const component = marker.component || ((_b = options.lineHighlight) == null ? void 0 : _b.component) || "div";
  const properties = __spreadValues({}, options);
  delete properties.lineHighlight;
  delete properties.markers;
  return {
    type: "element",
    tagName: component || "div",
    properties: marker.component ? Object.assign({}, properties, { className }) : { className },
    children,
    lineStart: marker.line,
    lineEnd: children[children.length - 1].lineEnd,
    isMarker: true
  };
}

// src/addMarkers/addMarkers.ts
function addMarkers(ast, options) {
  const markers = options.markers.map((marker) => {
    const finalMarker = typeof marker === "number" ? {
      line: marker
    } : marker;
    return __spreadValues(__spreadValues({}, finalMarker), options.lineHighlight);
  }).sort((nodeA, nodeB) => {
    return nodeA.line - nodeB.line;
  });
  const numbered = lineNumberify(ast).nodes;
  return wrapLines(numbered, markers, options);
}
function lineNumberify(ast, context = { lineNumber: 1 }) {
  return ast.reduce((result, node) => {
    const lineStart = context.lineNumber;
    if (node.type === "text") {
      if (node.value.indexOf("\n") === -1) {
        node.lineStart = lineStart;
        node.lineEnd = lineStart;
        result.nodes.push(node);
        return result;
      }
      const lines = node.value.split("\n");
      for (let i = 0; i < lines.length; i++) {
        const lineNum = i === 0 ? context.lineNumber : ++context.lineNumber;
        result.nodes.push({
          type: "text",
          value: i === lines.length - 1 ? lines[i] : `${lines[i]}
`,
          lineStart: lineNum,
          lineEnd: lineNum
        });
      }
      result.lineNumber = context.lineNumber;
      return result;
    }
    if (node.children) {
      const processed = lineNumberify(node.children, context);
      const firstChild = processed.nodes[0];
      const lastChild = processed.nodes[processed.nodes.length - 1];
      node.lineStart = firstChild ? firstChild.lineStart : lineStart;
      node.lineEnd = lastChild ? lastChild.lineEnd : lineStart;
      node.children = processed.nodes;
      result.lineNumber = processed.lineNumber;
      result.nodes.push(node);
      return result;
    }
    result.nodes.push(node);
    return result;
  }, { nodes: [], lineNumber: context.lineNumber });
}

// src/lang/svelte.ts
svelte.displayName = "svelte";
svelte.aliases = [];
function svelte(Prism) {
  const blocks = "(if|else if|await|then|catch|each|html|debug)";
  Prism.languages.svelte = Prism.languages.extend("markup", {
    each: {
      pattern: new RegExp("{[#/]each(?:(?:\\{(?:(?:\\{(?:[^{}])*\\})|(?:[^{}]))*\\})|(?:[^{}]))*}"),
      inside: {
        "language-javascript": [
          {
            pattern: /(as[\s\S]*)\([\s\S]*\)(?=\s*\})/,
            lookbehind: true,
            inside: Prism.languages["javascript"]
          },
          {
            pattern: /(as[\s]*)[\s\S]*(?=\s*)/,
            lookbehind: true,
            inside: Prism.languages["javascript"]
          },
          {
            pattern: /(#each[\s]*)[\s\S]*(?=as)/,
            lookbehind: true,
            inside: Prism.languages["javascript"]
          }
        ],
        keyword: /[#/]each|as/,
        punctuation: /{|}/
      }
    },
    block: {
      pattern: new RegExp("{[#:/@]/s" + blocks + "(?:(?:\\{(?:(?:\\{(?:[^{}])*\\})|(?:[^{}]))*\\})|(?:[^{}]))*}"),
      inside: {
        punctuation: /^{|}$/,
        keyword: [new RegExp("[#:/@]" + blocks + "( )*"), /as/, /then/],
        "language-javascript": {
          pattern: /[\s\S]*/,
          inside: Prism.languages["javascript"]
        }
      }
    },
    tag: {
      pattern: /<\/?(?!\d)[^\s>/=$<%]+(?:\s(?:\s*[^\s>/=]+(?:\s*=\s*(?:(?:"[^"]*"|'[^']*'|[^\s'">=]+(?=[\s>]))|(?:"[^"]*"|'[^']*'|{[\s\S]+?}(?=[\s/>])))|(?=[\s/>])))+)?\s*\/?>/i,
      greedy: true,
      inside: {
        tag: {
          pattern: /^<\/?[^\s>/]+/i,
          inside: {
            punctuation: /^<\/?/,
            namespace: /^[^\s>/:]+:/
          }
        },
        "language-javascript": {
          pattern: /\{(?:(?:\{(?:(?:\{(?:[^{}])*\})|(?:[^{}]))*\})|(?:[^{}]))*\}/,
          inside: Prism.languages["javascript"]
        },
        "attr-value": {
          pattern: /=\s*(?:"[^"]*"|'[^']*'|[^\s'">=]+)/i,
          inside: {
            punctuation: [
              /^=/,
              {
                pattern: /^(\s*)["']|["']$/,
                lookbehind: true
              }
            ],
            "language-javascript": {
              pattern: /{[\s\S]+}/,
              inside: Prism.languages["javascript"]
            }
          }
        },
        punctuation: /\/?>/,
        "attr-name": {
          pattern: /[^\s>/]+/,
          inside: {
            namespace: /^[^\s>/:]+:/
          }
        }
      }
    },
    "language-javascript": {
      pattern: /\{(?:(?:\{(?:(?:\{(?:[^{}])*\})|(?:[^{}]))*\})|(?:[^{}]))*\}/,
      lookbehind: true,
      inside: Prism.languages["javascript"]
    }
  });
  Prism.languages.svelte["tag"].inside["attr-value"].inside["entity"] = Prism.languages.svelte["entity"];
  Prism.hooks.add("wrap", (env) => {
    if (env.type === "entity") {
      env.attributes["title"] = env.content.replace(/&amp;/, "&");
    }
  });
  Object.defineProperty(Prism.languages.svelte.tag, "addInlined", {
    value: function addInlined(tagName, lang) {
      const includedCdataInside = {};
      includedCdataInside["language-" + lang] = {
        pattern: /(^<!\[CDATA\[)[\s\S]+?(?=\]\]>$)/i,
        lookbehind: true,
        inside: Prism.languages[lang]
      };
      includedCdataInside["cdata"] = /^<!\[CDATA\[|\]\]>$/i;
      const inside = {
        "included-cdata": {
          pattern: /<!\[CDATA\[[\s\S]*?\]\]>/i,
          inside: includedCdataInside
        }
      };
      inside["language-" + lang] = {
        pattern: /[\s\S]+/,
        inside: Prism.languages[lang]
      };
      const def = {};
      def[tagName] = {
        pattern: RegExp(/(<__[\s\S]*?>)(?:<!\[CDATA\[[\s\S]*?\]\]>\s*|[\s\S])*?(?=<\/__>)/.source.replace(/__/g, tagName), "i"),
        lookbehind: true,
        greedy: true,
        inside
      };
      Prism.languages.insertBefore("svelte", "cdata", def);
    }
  });
  Prism.languages.svelte.tag.addInlined("style", "css");
  Prism.languages.svelte.tag.addInlined("script", "javascript");
}

// src/mdxPrism/helpers.ts
import rangeParser from "parse-numeric-range";
var LANGUAGE_PREFIX = "language-";
var LINE_RANGE_REGEX = /{.*}/g;
function extractClassInformationFromNode(node) {
  const originalClassName = extractLanguageClassNameFromNode(node);
  if (!originalClassName) {
    return;
  }
  const normalizedClassName = normalizeLanguageClassName(originalClassName);
  return {
    originalClassName,
    languageClassName: normalizedClassName,
    language: extractLanguage(normalizedClassName),
    markers: extractLineRange(originalClassName)
  };
}
function normalizeLanguageClassName(className) {
  return className.replace(LINE_RANGE_REGEX, "").toLowerCase();
}
function extractLanguage(className) {
  return className.replace(LANGUAGE_PREFIX, "").replace(LINE_RANGE_REGEX, "");
}
function extractLanguageClassNameFromNode(node) {
  const classNames = node.properties.className || [];
  for (const className of classNames) {
    if (className.slice(0, 9) === LANGUAGE_PREFIX) {
      return className;
    }
  }
  return void 0;
}
function extractLineRange(className) {
  var _a;
  const [rawLines] = (_a = LINE_RANGE_REGEX.exec(className)) != null ? _a : [];
  if (!rawLines) {
    return;
  }
  const sanitizedLines = rawLines.replace("{", "").replace("}", "");
  return rangeParser(sanitizedLines);
}

// src/mdxPrism/mdxPrism.ts
refractor.register(svelte);
function mdxPrism(options = {}) {
  return function mdxPrism2Visit(tree) {
    visit(tree, "element", visitor);
  };
  function visitor(node, _index, parent) {
    if (!parent || parent.tagName !== "pre" || node.tagName !== "code") {
      return;
    }
    const classNameInfo = extractClassInformationFromNode(node);
    if (!classNameInfo) {
      return;
    }
    const { language, languageClassName, markers } = classNameInfo;
    let nextChildren;
    try {
      const parentClassNames = parent.properties.className || [];
      parent.properties.className = [...parentClassNames, languageClassName];
      nextChildren = refractor.highlight(toString(node), language).children;
      if (markers && markers.length > 0) {
        const PLAIN_TEXT_WITH_LF_TEST = /<span class="token plain-text">[^<]*\n[^<]*<\/span>/g;
        const html_ = rehype().stringify({ type: "root", children: nextChildren }).toString().replace(PLAIN_TEXT_WITH_LF_TEST, (match) => match.replace(/\n/g, '</span>\n<span class="token plain-text">'));
        const hast_ = unified().use(parse, { emitParseErrors: true, fragment: true }).parse(html_);
        nextChildren = addMarkers(hast_.children, {
          markers,
          lineHighlight: options.lineHighlight
        });
      }
    } catch (error) {
      if (options.ignoreMissing && /unknown language/gi.test(error.message)) {
        return;
      }
      throw error;
    }
    node.children = nextChildren;
  }
}

// src/types.ts
import { VisitorResult } from "unist-util-visit";
export {
  mdxPrism as default
};
